// Test program for the heap class



// Include files

#include "RNBasics/RNBasics.h"



// Data type definition

struct Data {
  RNScalar value;
  struct Data **heapentry;
};



int main()
{
    // Initialize library
    if (!RNInitBasics()) exit(-1); 

    // Create heap
    Data tmp;
    RNHeap<Data *> heap(&tmp, &tmp.value, &tmp.heapentry);

    // Perform a bunch of random pushes
    for (int i = 0; i < 100; i++) {
      Data *data = new Data();
      data->value = RNRandomScalar();
      heap.Push(data);
    }

    // Perform a bunch of random updates
    for (int i = 0; i < 50; i++) {
      int index = (int) (RNRandomScalar() * heap.NEntries());
      if (index >= heap.NEntries()) index--;
      Data *data = heap.Kth(index);
      data->value = RNRandomScalar();
      heap.Update(data);
    }

    // Perform a bunch of pops and random pushes
    for (int i = 0; i < 50; i++) {
      Data *data = heap.Pop();
      data->value = RNRandomScalar();
      heap.Push(data);
    }

    // Perform a bunch of random updates
    for (int i = 0; i < 50; i++) {
      int index = (int) (RNRandomScalar() * heap.NEntries());
      if (index >= heap.NEntries()) index--;
      Data *data = heap.Kth(index);
      data->value = RNRandomScalar();
      heap.Update(data);
    }

    // Print heap entries as they are popped
    RNScalar last = -1;
    RNBoolean error = FALSE;
    while (!heap.IsEmpty()) {
      Data *data = heap.Pop();
      printf("%g ", data->value); 
      if (data->value < last) { error = TRUE; break; }
      last = data->value;
    }

    // Print final message
    if (last == 0) printf("\nError -- random numbers not generated properly\n");
    else if (error) printf("\nError -- value is out of order\n");
    else printf("\nOK -- Heap is working properly\n");

    // Stop library
    RNStopBasics();

    // Return success
    return 0;
}















