// Include file for mesh class



////////////////////////////////////////////////////////////
// VERTEX CLASS DECLARATION
////////////////////////////////////////////////////////////

struct MeshVertex {
  // Constructors
  MeshVertex(void);
  MeshVertex(const R3Point& position);
  MeshVertex(double x, double y, double z);

  // Raw data
  R3Point position;
  int id;
};



////////////////////////////////////////////////////////////
// FACE CLASS DECLARATION
////////////////////////////////////////////////////////////

struct MeshFace {
  // Constructors
  MeshFace(void);
  MeshFace(MeshVertex *v1, MeshVertex *v2, MeshVertex *v3);
  MeshFace(const RNArray<MeshVertex *>& vertices);

  // Vertex access
  int NVertices(void);
  MeshVertex *Vertex(int k);

  // Raw data
  RNArray<MeshVertex *> vertices;
  R3Plane plane;
  int id;
};



////////////////////////////////////////////////////////////
// MESH CLASS DECLARATION
////////////////////////////////////////////////////////////

struct Mesh {
  // Constructors
  Mesh(void);

  // Vertex access
  int NVertices(void);
  MeshVertex *Vertex(int k);

  // Face access
  int NFaces(void);
  MeshFace *Face(int k);

  // File input/output 
  int ReadOffFile(const char *filename);
  int WriteOffFile(const char *filename);

  // Raw data
  RNArray<MeshVertex *> vertices;
  RNArray<MeshFace *> faces;
  R3Box bbox;
};




////////////////////////////////////////////////////////////
// INLINE FUNCTION DEFINITIONS
////////////////////////////////////////////////////////////

inline int MeshFace::
NVertices(void) 
{
  // Return number of vertices in face
  return vertices.NEntries();
}



inline MeshVertex *MeshFace::
Vertex(int k) 
{
  // Return the kth vertex in a face
  return vertices[k];
}



inline int Mesh::
NVertices(void) 
{
  // Return number of vertices in mesh
  return vertices.NEntries();
}



inline MeshVertex *Mesh::
Vertex(int k) 
{
  // Return the kth vertex in the mesh
  return vertices[k];
}



inline int Mesh::
NFaces(void) 
{
  // Return number of faces in mesh
  return faces.NEntries();
}



inline MeshFace *Mesh::
Face(int k) 
{
  // Return the kth face in the mesh
  return faces[k];
}




